@@ -2,8 +2,6 @@
 
 "$MAKE" || exit
 
-"$PERL_PREFIX"/bin/SP-patch-postinstall
-
 if find blib/arch/auto -type f \( -name "*.a" -o -name "*.obj" -o -name "*.lib" \) | grep -q .; then
    echo Probably a static XS module, rebuilding perl
    if "$MAKE" all perl; then
@@ -17,5 +15,7 @@ if find blib/arch/auto -type f \( -name "*.a" -o -name "*.obj" -o -name "*.lib"
    fi
 fi
 
-"$MAKE" install UNINST=1
+"$MAKE" install UNINST=1 || exit
+
+"$PERL_PREFIX"/bin/SP-patch-postinstall
 
